import json
from typing import Any

from .types import EnvironmentIdentifier
from .types import MeroxaApiResponse
from .utils import api_response
from .utils import ComplexEncoder

RESOURCE_BASE_PATH = "/v1/resources"


class Status(object):
    def __init__(
        self,
        state: str,
        details: str = None,
        last_updated_at: str = None,
    ) -> None:
        self.state = state
        self.details = details
        self.last_updated_at = last_updated_at


class ResourcesResponse(MeroxaApiResponse):
    def __init__(
        self,
        uuid: str,
        name: str,
        type: str,
        url: str,
        metadata: dict,
        connector_count: int,
        status: Any,
        created_at: str,
        updated_at: str,
        **kwargs
    ) -> None:
        self.uuid = uuid
        self.name = name
        self.type = type
        self.url = url
        self.metadata = metadata
        self.connector_count = connector_count
        self.status = Status(**status)
        self.created_at = created_at
        self.updated_at = updated_at
        super().__init__()


class ResourceCredentials:
    def __init__(
        self,
        username: str,
        password: str,
        ca_cert: str,
        client_cert: str,
        client_cert_key: str,
        ssl: bool,
    ):
        self._username = username
        self._password = password
        self._ca_cert = ca_cert
        self._client_cert = client_cert
        self._client_cert_key = client_cert_key
        self._ssl = ssl

    def repr_json(self):
        return dict(
            username=self._username,
            password=self._password,
            ca_cert=self._ca_cert,
            client_cert=self._client_cert,
            client_cert_key=self._client_cert_key,
            ssl=self._ssl,
        )


class ResourceSSHTunnel:
    def __init__(self, address: str, private_key: str):
        self._address = address
        self._private_key = private_key

    def repr_json(self):
        return dict(address=self.address, private_key=self.private_key)


class CreateResourceParams:
    def __init__(
        self,
        url: str,
        type: str,
        name: str = None,
        metadata: dict[str, str] = None,
        credentials: ResourceCredentials = None,
        environment: EnvironmentIdentifier = None,
        ssh_tunnel: ResourceSSHTunnel = None,
    ) -> None:
        self._url = url
        self._type = type
        self._name = name
        self._credentials = credentials
        self._metadata = metadata
        self._environment = environment
        self._ssh_tunnel = ssh_tunnel

    def repr_json(self):
        return dict(
            name=self._name,
            url=self._url,
            credentials=self._credentials,
            environment=self._environment,
            metadata=self._metadata,
            type=self._type,
            ssh_tunnel=self._ssh_tunnel,
        )


class UpdateResourceParams:
    def __init__(
        self,
        name: str,
        url: str,
        credentials: ResourceCredentials = None,
        metadata: dict[str, str] = None,
        ssh_tunnel: ResourceSSHTunnel = None,
    ) -> None:
        self._name = name
        self._url = url
        self._credentials = credentials
        self._metadata = metadata
        self._ssh_tunnel = ssh_tunnel

    def repr_json(self):
        return dict(
            name=self._name,
            url=self._url,
            credentials=self._credentials,
            metadata=self._metadata,
            ssh_tunnel=self._ssh_tunnel,
        )


class Resources:
    def __init__(self, session) -> None:
        self._session = session

    @api_response(ResourcesResponse)
    async def get(self, name_or_id: str):
        async with self._session.get(
            RESOURCE_BASE_PATH + "/{}".format(name_or_id)
        ) as resp:
            return await resp.text()

    @api_response(ResourcesResponse)
    async def list(self):
        async with self._session.get(RESOURCE_BASE_PATH) as resp:
            return await resp.text()

    async def delete(self, name_or_id: str):
        async with self._session.delete(
            RESOURCE_BASE_PATH + "/{}".format(name_or_id)
        ) as resp:
            return await resp.text()

    @api_response(ResourcesResponse)
    async def create(self, create_resource_parameters: CreateResourceParams):
        async with self._session.post(
            RESOURCE_BASE_PATH,
            data=json.dumps(create_resource_parameters.repr_json(), cls=ComplexEncoder),
        ) as resp:
            return await resp.text()

    @api_response(ResourcesResponse)
    async def update(
        self, name_or_id: str, update_resource_parameters: UpdateResourceParams
    ):
        async with self._session.post(
            RESOURCE_BASE_PATH + "/{}".format(name_or_id),
            json=json.dumps(update_resource_parameters.repr_json(), cls=ComplexEncoder),
        ) as resp:
            res = await resp.text()
            return ResourcesResponse(**json.loads(res))



from enum import Enum
from typing import Any


class ResourceType(Enum):
    POSTGRES = "postgres"
    MYSQL = "mysql"
    REDSHIFT = "redshift"
    URL = "url"
    S3 = "s3"
    MONGODB = "mongodb"
    ELASTICSEARCH = "elasticsearch"
    SNOWFLAKE = "snowflakedb"
    BIGQUERY = "bigquery"
    SQLSERVER = "sqlserver"
    COSMODB = "cosmodb"
    KAFKA = "kafka"
    CONFLUENTCLOUD = "confluentcloud"


class MeroxaApiResponse(object):
    def __init__(self, *args, **kwargs):
        ...


class EnvironmentIdentifier:
    def __init__(self, name=None, uuid=None):
        self.name = name
        self.uuid = uuid

    def repr_json(self):
        return dict(name=self.name) if self.name is not None else dict(uuid=self.uuid)


class ResourceCollection:
    def __init__(self, name=None, destination=None, source=None):

        self.name = name
        self.source = source
        self.destination = destination


class ApplicationResource:
    def __init__(self, name=None, uuid=None, collection: Any = None):

        self.name = name
        self.uuid = uuid
        self.collection = ResourceCollection(**collection)


class EntityIdentifier:
    def __init__(self, name=None, uuid=None):
        self.name = name
        self.uuid = uuid

    def repr_json(self):
        return dict(name=self.name) if self.name is not None else dict(uuid=self.uuid)



from .applications import ApplicationResponse
from .applications import Applications
from .applications import CreateApplicationParams
from .client import Meroxa
from .connectors import Connectors
from .connectors import ConnectorsResponse
from .connectors import CreateConnectorParams
from .connectors import UpdateConnectorParams
from .functions import CreateFunctionParams
from .functions import FunctionResponse
from .functions import Functions
from .pipelines import CreatePipelineParams
from .pipelines import PipelineIdentifiers
from .pipelines import PipelineResponse
from .pipelines import UpdatePipelineParams
from .resources import CreateResourceParams
from .resources import ResourceCredentials
from .resources import Resources
from .resources import ResourcesResponse
from .resources import ResourceSSHTunnel
from .resources import UpdateResourceParams
from .types import EnvironmentIdentifier
from .types import ResourceType
from .users import UserResponse
from .users import Users
from .utils import ComplexEncoder
from .utils import ErrorResponse

__all__ = [
    "Applications",
    "ApplicationResponse",
    "ComplexEncoder",
    "Connectors",
    "ConnectorsResponse",
    "CreateApplicationParams",
    "CreateConnectorParams",
    "CreateFunctionParams",
    "CreatePipelineParams",
    "CreateResourceParams",
    "EnvironmentIdentifier",
    "ErrorResponse",
    "FunctionResponse",
    "Functions",
    "Meroxa",
    "PipelineIdentifiers",
    "PipelineResponse",
    "ResourceCredentials",
    "Resources",
    "ResourcesResponse",
    "ResourceSSHTunnel",
    "ResourceType",
    "UpdateConnectorParams",
    "UpdatePipelineParams",
    "UpdateResourceParams",
    "UserResponse",
    "Users",
]



import json

from .pipelines import PipelineIdentifiers
from .types import MeroxaApiResponse
from .utils import api_response
from .utils import ComplexEncoder

FUNCTIONS_BASE_PATH = "/v1/functions"


class FunctionResponse(MeroxaApiResponse):
    def __init__(
        self,
        uuid: str,
        name: str,
        input_stream: str,
        output_stream: str,
        image: str,
        command: list[str],
        args: list[str],
        env_vars: dict[str, str],
        status: dict,
        pipeline: dict,
    ) -> None:
        self.uuid = uuid
        self.name = name
        self.input_stream = input_stream
        self.output_stream = output_stream
        self.image = image
        self.command = command
        self.args = args
        self.env_vars = env_vars
        self.status = status
        self.pipeline = pipeline
        super().__init__()


class CreateFunctionParams:
    def __init__(
        self,
        name: str,
        input_stream: str,
        output_stream: str,
        pipeline: PipelineIdentifiers,
        image: str,
        command: list[str],
        args: list[str],
        env_vars: dict[str, str],
    ) -> None:
        self._name = name
        self._input_stream = input_stream
        self._output_stream = output_stream
        self._image = image
        self._command = command
        self._args = args
        self._env_vars = env_vars
        self._pipeline = pipeline

    def repr_json(self):
        return dict(
            name=self._name,
            input_stream=self._input_stream,
            output_stream=self._output_stream,
            image=self._image,
            command=self._command,
            args=self._args,
            pipeline=self._pipeline,
            env_vars=self._env_vars,
        )


class Functions:
    def __init__(self, session) -> None:
        self._session = session

    @api_response(FunctionResponse)
    async def get(self, name_or_id: str):
        async with self._session.get(
            FUNCTIONS_BASE_PATH + "/{}".format(name_or_id)
        ) as resp:
            return await resp.text()

    @api_response(FunctionResponse)
    async def list(self):
        async with self._session.get(FUNCTIONS_BASE_PATH) as resp:
            return await resp.text()

    async def delete(self, name_or_id: str):
        async with self._session.delete(
            FUNCTIONS_BASE_PATH + "/{}".format(name_or_id)
        ) as resp:
            return await resp.text()

    @api_response(FunctionResponse)
    async def create(self, create_function_parameters: CreateFunctionParams):
        async with self._session.post(
            FUNCTIONS_BASE_PATH,
            data=json.dumps(create_function_parameters.repr_json(), cls=ComplexEncoder),
        ) as resp:
            return await resp.text()
